module Gogdb
  
  class Cli < Thor
    def initialize(*args)
      check_config
      super
    end

    package_name "Gogdb"
    map "--version" => :version

    desc "version", "Shows current version"
    def version
      puts Gogdb::VERSION
    end

    desc "fetch [options]", "Fetches data and updates database"
    method_option :limit, :type => :numeric, :desc => "How many items you want to fetch. 0 means no limit.", :default => 0
    method_option :requestLimit, :type => :numeric, :desc => "Limit requests per minute. 0 means no limit.", :default => 12
    method_option :type, :type => :string, :enum => ["all", "games", "movies"], :desc => "What type of items you want to fetch.", :default => "all"
    method_option :debug, :type => :boolean, :desc => "Enables debug messages. Useful if you want to see what's happening step-by-step.", :default => false
    method_option :silent, :type => :boolean, :desc => "Hides all output. Even debug messages.", :default => false
    def fetch
      @e = Gogdb::Engine.new(options)
      @e.fetch(options)
    end

    desc "setup", "Run automatic configuration."
    def setup
      db_path = ask("Enter path you want to save database (default #{ENV['HOME']}/.gogdb):")

      db_path = "#{ENV['HOME']}/.gogdb" if db_path.empty?

      @config = {
        "path" => db_path
      }

      # Stop execution if user does not want to overwrite.
      if File.exist?("config.yml")
        exit unless yes? "Config file already exists. Overwrite?(y/n)", :red
      end
      
      File.open("config.yml", "w") { |file|
        file.write(@config.to_yaml)
      }

      say "Config written successfully.", :green
    end

    private

    # Checks if config file exists. If not, runs setup automatically.
    def check_config
      unless File.exist?("config.yml")
        say "It appears that you don't have config in place. You need to enter following details to make Gogdb work;", :yellow
        setup
      end
    end

  end
end
